import { Component, ElementRef, Input, HostListener, Directive, Inject, forwardRef, ContentChild, TemplateRef, ViewContainerRef, Renderer2, NgModule, EventEmitter, Output, QueryList, ContentChildren, ViewChild, HostBinding } from '@angular/core';
import { BehaviorSubject, fromEvent } from 'rxjs';
import { distinctUntilChanged, skip, first } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { __awaiter } from 'tslib';

function assertNever(x) {
    throw new Error('Unexpected object: ' + x);
}
var Focus;
(function (Focus) {
    /** Focus the first non-disabled item. */
    Focus[Focus["First"] = 0] = "First";
    /** Focus the previous non-disabled item. */
    Focus[Focus["Previous"] = 1] = "Previous";
    /** Focus the next non-disabled item. */
    Focus[Focus["Next"] = 2] = "Next";
    /** Focus the last non-disabled item. */
    Focus[Focus["Last"] = 3] = "Last";
    /** Focus a specific item based on the `id` of the item. */
    Focus[Focus["Specific"] = 4] = "Specific";
    /** Focus no items at all. */
    Focus[Focus["Nothing"] = 5] = "Nothing";
})(Focus || (Focus = {}));
;
function calculateActiveIndex(action, resolvers) {
    const items = resolvers.resolveItems();
    if (items.length <= 0)
        return null;
    const currentActiveIndex = resolvers.resolveActiveIndex();
    const activeIndex = currentActiveIndex !== null && currentActiveIndex !== void 0 ? currentActiveIndex : -1;
    const nextActiveIndex = (() => {
        switch (action.focus) {
            case Focus.First:
                return items.findIndex(item => !resolvers.resolveDisabled(item));
            case Focus.Previous: {
                const idx = items
                    .slice()
                    .reverse()
                    .findIndex((item, idx, all) => {
                    if (activeIndex !== -1 && all.length - idx - 1 >= activeIndex)
                        return false;
                    return !resolvers.resolveDisabled(item);
                });
                if (idx === -1)
                    return idx;
                return items.length - 1 - idx;
            }
            case Focus.Next:
                return items.findIndex((item, idx) => {
                    if (idx <= activeIndex)
                        return false;
                    return !resolvers.resolveDisabled(item);
                });
            case Focus.Last: {
                const idx = items
                    .slice()
                    .reverse()
                    .findIndex(item => !resolvers.resolveDisabled(item));
                if (idx === -1)
                    return idx;
                return items.length - 1 - idx;
            }
            case Focus.Specific:
                return items.findIndex(item => resolvers.resolveId(item) === action.id);
            case Focus.Nothing:
                return null;
            default:
                assertNever(action);
        }
    })();
    return nextActiveIndex === -1 ? currentActiveIndex : nextActiveIndex;
}

// TODO: This must already exist somewhere, right? 🤔
// Ref: https://www.w3.org/TR/uievents-key/#named-key-attribute-values
var Keys;
(function (Keys) {
    Keys["Space"] = " ";
    Keys["Enter"] = "Enter";
    Keys["Escape"] = "Escape";
    Keys["Backspace"] = "Backspace";
    Keys["ArrowUp"] = "ArrowUp";
    Keys["ArrowDown"] = "ArrowDown";
    Keys["Home"] = "Home";
    Keys["End"] = "End";
    Keys["PageUp"] = "PageUp";
    Keys["PageDown"] = "PageDown";
    Keys["Tab"] = "Tab";
})(Keys || (Keys = {}));
;

let id = 0;
function generateId() {
    return ++id;
}
function useId() {
    return generateId();
}

var MenuStates;
(function (MenuStates) {
    MenuStates[MenuStates["Closed"] = 0] = "Closed";
    MenuStates[MenuStates["Open"] = 1] = "Open";
})(MenuStates || (MenuStates = {}));
;

class MenuComponent {
    constructor(elementRef) {
        this.elementRef = elementRef;
        this.static = false;
        this.menuState = new BehaviorSubject(MenuStates.Closed);
        this.buttonRef = null;
        this.itemsRef = null;
        this.items = [];
        this.searchQuery = '';
        this.activeItemIndex = null;
    }
    closeMenu() {
        this.menuState.next(MenuStates.Closed);
        this.activeItemIndex = null;
    }
    openMenu() {
        this.menuState.next(MenuStates.Open);
    }
    isOpened() {
        return this.menuState && this.menuState.value === MenuStates.Open;
    }
    goToItem(focus, id) {
        const nextActiveItemIndex = calculateActiveIndex(focus === Focus.Specific ? { focus: Focus.Specific, id: id } : { focus: focus }, {
            resolveItems: () => this.items,
            resolveActiveIndex: () => this.activeItemIndex,
            resolveId: item => item.id,
            resolveDisabled: item => item.dataRef.disabled,
        });
        if (this.searchQuery === '' && this.activeItemIndex === nextActiveItemIndex)
            return;
        this.searchQuery = '';
        this.activeItemIndex = nextActiveItemIndex;
        let elementId = null;
        if (this.activeItemIndex !== null) {
            elementId = this.items[this.activeItemIndex].id;
        }
    }
    search(value) {
        this.searchQuery += value;
        const match = this.items.findIndex(item => item.dataRef.textValue.startsWith(this.searchQuery) && !item.dataRef.disabled);
        if (match === -1 || match === this.activeItemIndex)
            return;
        this.activeItemIndex = match;
    }
    clearSearch() {
        this.searchQuery = '';
    }
    registerItem(id, dataRef) {
        this.items.push({ id, dataRef });
    }
    unregisterItem(id) {
        const nextItems = this.items.slice();
        const currentActiveItem = this.activeItemIndex !== null ? nextItems[this.activeItemIndex] : null;
        const idx = nextItems.findIndex(a => a.id === id);
        if (idx !== -1)
            nextItems.splice(idx, 1);
        this.items = nextItems;
        this.activeItemIndex = (() => {
            if (idx === this.activeItemIndex)
                return null;
            if (currentActiveItem === null)
                return null;
            // If we removed the item before the actual active index, then it would be out of sync. To
            // fix this, we will find the correct (new) index position.
            return nextItems.indexOf(currentActiveItem);
        })();
    }
    handler(event) {
        var _a, _b, _c;
        const target = event.target;
        const active = document.activeElement;
        if (this.menuState.value !== MenuStates.Open)
            return;
        if ((_a = this.buttonRef) === null || _a === void 0 ? void 0 : _a.contains(target))
            return;
        if (!((_b = this.elementRef.nativeElement) === null || _b === void 0 ? void 0 : _b.contains(target)))
            this.closeMenu();
        if (active !== document.body && (active === null || active === void 0 ? void 0 : active.contains(target)))
            return; // Keep focus on newly clicked/focused element
        if (!event.defaultPrevented)
            (_c = this.buttonRef) === null || _c === void 0 ? void 0 : _c.focus({ preventScroll: true });
    }
}
MenuComponent.decorators = [
    { type: Component, args: [{
                selector: '[Menu], [HeadlessMenu]',
                template: `<ng-content></ng-content>`
            },] }
];
MenuComponent.ctorParameters = () => [
    { type: ElementRef }
];
MenuComponent.propDecorators = {
    static: [{ type: Input }],
    handler: [{ type: HostListener, args: ['window:click', ['$event'],] }]
};

function nextFrame(cb) {
    requestAnimationFrame(() => requestAnimationFrame(cb));
}
class MenuButtonDirective {
    constructor(menu, elementRef) {
        this.menu = menu;
        this.elementRef = elementRef;
        this.disabled = false;
        this.id = `headlessui-menu-button-${useId()}`;
    }
    ngAfterViewInit() {
        this.menu.buttonRef = this.elementRef.nativeElement;
    }
    isOpen() {
        return this.menu.menuState.value === MenuStates.Open ? true : undefined;
    }
    handleKeyDown(event) {
        switch (event.key) {
            // Ref: https://www.w3.org/TR/wai-aria-practices-1.2/#keyboard-interaction-13
            case Keys.Space:
            case Keys.Enter:
            case Keys.ArrowDown:
                event.preventDefault();
                this.menu.openMenu();
                setTimeout(() => {
                    var _a;
                    (_a = this.menu.itemsRef) === null || _a === void 0 ? void 0 : _a.focus({ preventScroll: true });
                    this.menu.goToItem(Focus.First);
                });
                break;
            case Keys.ArrowUp:
                event.preventDefault();
                this.menu.openMenu();
                setTimeout(() => {
                    var _a;
                    (_a = this.menu.itemsRef) === null || _a === void 0 ? void 0 : _a.focus({ preventScroll: true });
                    this.menu.goToItem(Focus.Last);
                });
                break;
        }
    }
    handlePointerUp(event) {
        if (this.disabled)
            return;
        if (this.menu.isOpened()) {
            this.menu.closeMenu();
            setTimeout(() => { var _a; return (_a = this.menu.buttonRef) === null || _a === void 0 ? void 0 : _a.focus({ preventScroll: true }); });
        }
        else {
            event.preventDefault();
            this.menu.openMenu();
            nextFrame(() => { var _a; return (_a = this.menu.itemsRef) === null || _a === void 0 ? void 0 : _a.focus({ preventScroll: true }); });
        }
    }
}
MenuButtonDirective.decorators = [
    { type: Directive, args: [{
                selector: '[MenuButton], [HeadlessMenuButton]',
                host: {
                    '[attr.id]': 'id',
                    '[attr.type]': '"button"',
                    '[attr.aria-haspopup]': 'true',
                    // '[attr.aria-controls]': 'menu.itemsRef.id',
                    '[attr.aria-expanded]': 'isOpen()',
                }
            },] }
];
MenuButtonDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [forwardRef(() => MenuComponent),] }] },
    { type: ElementRef }
];
MenuButtonDirective.propDecorators = {
    disabled: [{ type: Input }],
    handleKeyDown: [{ type: HostListener, args: ['keydown', ['$event'],] }],
    handlePointerUp: [{ type: HostListener, args: ['pointerup', ['$event'],] }]
};

class MenuItemComponent {
    constructor(menu, elementRef) {
        this.menu = menu;
        this.elementRef = elementRef;
        this.id = `headlessui-menu-item-${useId()}`;
        this.dataRef = { textValue: '', disabled: this._disabled };
    }
    set disabled(value) {
        if (value || value === '') {
            this._disabled = true;
        }
        else {
            this._disabled = false;
        }
    }
    ngOnDestroy() {
        this.menu.unregisterItem(this.id);
    }
    ngAfterViewInit() {
        var _a, _b;
        const textValue = (_b = (_a = this.elementRef.nativeElement) === null || _a === void 0 ? void 0 : _a.textContent) === null || _b === void 0 ? void 0 : _b.toLowerCase().trim();
        if (textValue !== undefined)
            this.dataRef.textValue = textValue;
        this.menu.registerItem(this.id, this.dataRef);
    }
    get active() {
        return this.menu.activeItemIndex !== null ? this.menu.items[this.menu.activeItemIndex].id === this.id : false;
    }
    handleClick(event) {
        if (this.disabled)
            return event.preventDefault();
        this.menu.closeMenu();
        setTimeout(() => { var _a; return (_a = this.menu.buttonRef) === null || _a === void 0 ? void 0 : _a.focus({ preventScroll: true }); });
    }
    handleFocus() {
        if (this.disabled)
            return this.menu.goToItem(Focus.Nothing);
        this.menu.goToItem(Focus.Specific, this.id);
    }
    handlePointerMove() {
        if (this.disabled)
            return;
        if (this.active)
            return;
        this.menu.goToItem(Focus.Specific, this.id);
    }
    handlePointerLeave() {
        if (this.disabled)
            return;
        if (!this.active)
            return;
        this.menu.goToItem(Focus.Nothing);
    }
}
MenuItemComponent.decorators = [
    { type: Component, args: [{
                selector: '[MenuItem], [HeadlessMenuItem]',
                host: {
                    '[attr.id]': 'id',
                    '[attr.role]': '"menuitem"',
                    '[attr.tabIndex]': '"-1"',
                    '[attr.aria-disabled]': '_disabled === true ? true : undefined'
                },
                template: `
    <ng-template [ngTemplateOutlet]="template" [ngTemplateOutletContext]="{$implicit: dataRef, 'active': active, 'disabled': _disabled}"></ng-template>
    <ng-content></ng-content>
  `
            },] }
];
MenuItemComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [forwardRef(() => MenuComponent),] }] },
    { type: ElementRef }
];
MenuItemComponent.propDecorators = {
    template: [{ type: ContentChild, args: [TemplateRef,] }],
    disabled: [{ type: Input }],
    handleClick: [{ type: HostListener, args: ['click', ['$event'],] }],
    handleFocus: [{ type: HostListener, args: ['focus',] }],
    handlePointerMove: [{ type: HostListener, args: ['pointermove',] }],
    handlePointerLeave: [{ type: HostListener, args: ['pointerleave',] }]
};

class MenuItemsDirective {
    constructor(menu, templateRef, viewContainer, renderer) {
        this.menu = menu;
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.renderer = renderer;
        this.id = `headlessui-menu-items-${useId()}`;
        this.searchDebounce = null;
    }
    ngAfterViewInit() {
        if (this.menu.static) {
            this.setAttributes();
        }
    }
    ngOnInit() {
        if (this.menu.static) {
            this.embedTemplate();
            return;
        }
        this.menu.menuState.pipe(distinctUntilChanged()).subscribe((menuState) => {
            this.hide();
            if (menuState === MenuStates.Open) {
                this.show();
            }
        });
    }
    show() {
        // If the menu is static don't modify anything let the user modify it's state
        if (this.menu.static) {
            return;
        }
        this.embedTemplate();
        this.setAttributes();
    }
    hide() {
        // If the menu is static don't modify anything let the user modify it's state
        if (this.menu.static) {
            return;
        }
        this.viewContainer.clear();
    }
    embedTemplate() {
        let view = this.viewContainer.createEmbeddedView(this.templateRef);
        this.elementRef = view.rootNodes[0];
        this.menu.itemsRef = this.elementRef;
    }
    setAttributes() {
        var _a, _b;
        this.renderer.setAttribute(this.elementRef, 'aria-activedescendant', `${this.menu.activeItemIndex === null ? undefined : (_a = this.menu.items[this.menu.activeItemIndex]) === null || _a === void 0 ? void 0 : _a.id}`);
        this.renderer.setAttribute(this.elementRef, 'aria-labelledby', `${(_b = this.menu.buttonRef) === null || _b === void 0 ? void 0 : _b.id}`);
        this.renderer.setAttribute(this.elementRef, 'id', this.id);
        this.renderer.setAttribute(this.elementRef, 'role', 'menu');
        this.renderer.setAttribute(this.elementRef, 'tabIndex', '0');
        fromEvent(this.elementRef, 'keydown').subscribe((event) => {
            var _a;
            if (this.searchDebounce)
                clearTimeout(this.searchDebounce);
            switch (event.key) {
                // Ref: https://www.w3.org/TR/wai-aria-practices-1.2/#keyboard-interaction-12
                // @ts-expect-error Fallthrough is expected here
                case Keys.Space:
                    if (this.menu.searchQuery !== '') {
                        event.preventDefault();
                        return this.menu.search(event.key);
                    }
                // When in type ahead mode, fallthrough
                case Keys.Enter:
                    event.preventDefault();
                    if (this.menu.activeItemIndex !== null) {
                        const { id } = this.menu.items[this.menu.activeItemIndex];
                        (_a = document.getElementById(id)) === null || _a === void 0 ? void 0 : _a.click();
                    }
                    this.menu.closeMenu();
                    setTimeout(() => { var _a; return (_a = this.menu.buttonRef) === null || _a === void 0 ? void 0 : _a.focus({ preventScroll: true }); });
                    break;
                case Keys.ArrowDown:
                    event.preventDefault();
                    return this.menu.goToItem(Focus.Next);
                case Keys.ArrowUp:
                    event.preventDefault();
                    return this.menu.goToItem(Focus.Previous);
                case Keys.Home:
                case Keys.PageUp:
                    event.preventDefault();
                    return this.menu.goToItem(Focus.First);
                case Keys.End:
                case Keys.PageDown:
                    event.preventDefault();
                    return this.menu.goToItem(Focus.Last);
                case Keys.Escape:
                    event.preventDefault();
                    this.menu.closeMenu();
                    setTimeout(() => { var _a; return (_a = this.menu.buttonRef) === null || _a === void 0 ? void 0 : _a.focus({ preventScroll: true }); });
                    break;
                case Keys.Tab:
                    return event.preventDefault();
                default:
                    if (event.key.length === 1) {
                        this.menu.search(event.key);
                        this.searchDebounce = setTimeout(() => this.menu.clearSearch(), 350);
                    }
                    break;
            }
        });
    }
}
MenuItemsDirective.decorators = [
    { type: Directive, args: [{
                selector: '[MenuItems], [HeadlessMenuItems]',
            },] }
];
MenuItemsDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [forwardRef(() => MenuComponent),] }] },
    { type: TemplateRef },
    { type: ViewContainerRef },
    { type: Renderer2 }
];

class MenuModule {
}
MenuModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    MenuComponent,
                    MenuItemsDirective,
                    MenuItemComponent,
                    MenuButtonDirective
                ],
                imports: [
                    CommonModule
                ],
                exports: [
                    MenuComponent,
                    MenuItemsDirective,
                    MenuItemComponent,
                    MenuButtonDirective
                ],
            },] }
];

class SwitchGroupDirective {
    constructor() {
        this.dataRef = { switchRef: null, labelRef: null, descriptionRef: null };
    }
}
SwitchGroupDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngx-switch-group], [switch-group], [SwitchGroup]',
            },] }
];
SwitchGroupDirective.ctorParameters = () => [];

class SwitchDescriptionDirective {
    constructor(switchGroup, elementRef) {
        this.switchGroup = switchGroup;
        this.id = `headlessui-switch-description-${useId()}`;
        this.switchGroup.dataRef.labelRef = elementRef.nativeElement;
    }
    handlePointerUp() {
        var _a, _b;
        (_a = this.switchGroup.dataRef.switchRef) === null || _a === void 0 ? void 0 : _a.click();
        (_b = this.switchGroup.dataRef.switchRef) === null || _b === void 0 ? void 0 : _b.focus({ preventScroll: true });
    }
}
SwitchDescriptionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngx-switch-description], [switch-description], [SwitchDescription]',
                host: {
                    '[attr.id]': 'id'
                }
            },] }
];
SwitchDescriptionDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [forwardRef(() => SwitchGroupDirective),] }] },
    { type: ElementRef }
];
SwitchDescriptionDirective.propDecorators = {
    handlePointerUp: [{ type: HostListener, args: ['pointerup',] }]
};

class SwitchLabelDirective {
    constructor(switchGroup, elementRef) {
        this.switchGroup = switchGroup;
        this.id = `headlessui-switch-label-${useId()}`;
        this.switchGroup.dataRef.labelRef = elementRef.nativeElement;
    }
    handlePointerUp() {
        var _a, _b;
        (_a = this.switchGroup.dataRef.switchRef) === null || _a === void 0 ? void 0 : _a.click();
        (_b = this.switchGroup.dataRef.switchRef) === null || _b === void 0 ? void 0 : _b.focus({ preventScroll: true });
    }
}
SwitchLabelDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngx-switch-label], [switch-label], [SwitchLabel]',
                host: {
                    '[attr.id]': 'id'
                }
            },] }
];
SwitchLabelDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [forwardRef(() => SwitchGroupDirective),] }] },
    { type: ElementRef }
];
SwitchLabelDirective.propDecorators = {
    handlePointerUp: [{ type: HostListener, args: ['pointerup',] }]
};

class SwitchComponent {
    constructor(switchGroup, elementRef) {
        this.switchGroup = switchGroup;
        this.id = `headlessui-switch-${useId()}`;
        this.value = false;
        this.change = new EventEmitter();
        this.propagateChange = (_) => { };
        this.switchGroup.dataRef.switchRef = elementRef.nativeElement;
    }
    toggle() {
        this.value = !this.value;
        this.change.emit(this.value);
    }
    handleClick(event) {
        event.preventDefault();
        this.toggle();
    }
    handleKeyUp(event) {
        if (event.key !== Keys.Tab)
            event.preventDefault();
        if (event.key === Keys.Space)
            this.toggle();
    }
    handleKeyPress(event) {
        event.preventDefault();
    }
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(fn) {
    }
}
SwitchComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-switch, switch, [Switch]',
                template: "<ng-template [ngTemplateOutlet]=\"templateRef\" [ngTemplateOutletContext]=\"{'checked': value}\"></ng-template>\n<ng-content></ng-content>\n",
                host: {
                    '[attr.id]': 'id',
                    '[attr.role]': '"switch"',
                    '[attr.tabIndex]': '0',
                    '[attr.aria-checked]': 'value',
                    '[attr.aria-labelledby]': 'switchGroup.dataRef.labelRef?.id',
                    '[attr.aria-describedby]': 'switchGroup.dataRef.descriptionRef?.id',
                },
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => SwitchComponent),
                        multi: true
                    }
                ]
            },] }
];
SwitchComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [forwardRef(() => SwitchGroupDirective),] }] },
    { type: ElementRef }
];
SwitchComponent.propDecorators = {
    templateRef: [{ type: ContentChild, args: [TemplateRef,] }],
    change: [{ type: Output }],
    handleClick: [{ type: HostListener, args: ['click', ['$event'],] }],
    handleKeyUp: [{ type: HostListener, args: ['onkeyup', ['$event'],] }],
    handleKeyPress: [{ type: HostListener, args: ['onkeydown', ['$event'],] }]
};

class SwitchModule {
}
SwitchModule.decorators = [
    { type: NgModule, args: [{
                declarations: [SwitchComponent, SwitchGroupDirective, SwitchLabelDirective, SwitchDescriptionDirective],
                imports: [
                    CommonModule
                ],
                exports: [SwitchComponent, SwitchGroupDirective, SwitchLabelDirective, SwitchDescriptionDirective]
            },] }
];

/**
 * Credit: https://github.com/tailwindlabs/headlessui/
 */
function disposables() {
    const disposables = [];
    const api = {
        requestAnimationFrame(...args) {
            const raf = requestAnimationFrame(...args);
            api.add(() => cancelAnimationFrame(raf));
        },
        nextFrame(...args) {
            api.requestAnimationFrame(() => {
                api.requestAnimationFrame(...args);
            });
        },
        setTimeout(...args) {
            const timer = setTimeout(...args);
            api.add(() => clearTimeout(timer));
        },
        add(cb) {
            disposables.push(cb);
        },
        dispose() {
            disposables.splice(0).forEach(dispose => dispose());
        },
    };
    return api;
}

/**
 * Credit: https://github.com/tailwindlabs/headlessui/
 */
function once(cb) {
    const state = { called: false };
    return (...args) => {
        if (state.called) {
            return;
        }
        state.called = true;
        return cb(...args);
    };
}

function addClasses(node, ...classes) {
    node && classes.length > 0 && node.classList.add(...classes);
}
function removeClasses(node, ...classes) {
    node && classes.length > 0 && node.classList.remove(...classes);
}
var Reason;
(function (Reason) {
    Reason["Finished"] = "finished";
    Reason["Cancelled"] = "cancelled";
})(Reason || (Reason = {}));
function waitForTransition(node, done) {
    const d = disposables();
    if (!node)
        return d.dispose;
    // Safari returns a comma separated list of values, so let's sort them and take the highest value.
    const { transitionDuration, transitionDelay } = getComputedStyle(node);
    const [durationMs, delaysMs] = [transitionDuration, transitionDelay].map(value => {
        const [resolvedValue = 0] = value
            .split(',')
            // Remove falseys we can't work with
            .filter(Boolean)
            // Values are returned as `0.3s` or `75ms`
            .map(v => (v.includes('ms') ? parseFloat(v) : parseFloat(v) * 1000))
            .sort((a, z) => z - a);
        return resolvedValue;
    });
    // Waiting for the transition to end. We could use the `transitionend` event, however when no
    // actual transition/duration is defined then the `transitionend` event is not fired.
    //
    // TODO: Downside is, when you slow down transitions via devtools this timeout is still using the
    // full 100% speed instead of the 25% or 10%.
    if (durationMs !== 0) {
        d.setTimeout(() => {
            done(Reason.Finished);
        }, durationMs + delaysMs);
    }
    else {
        // No transition is happening, so we should cleanup already. Otherwise we have to wait until we
        // get disposed.
        done(Reason.Finished);
    }
    // If we get disposed before the timeout runs we should cleanup anyway
    d.add(() => done(Reason.Cancelled));
    return d.dispose;
}
function transition(node, base, from, to, done) {
    const d = disposables();
    const _done = done !== undefined ? once(done) : () => { };
    addClasses(node, ...base, ...from);
    d.nextFrame(() => {
        removeClasses(node, ...from);
        addClasses(node, ...to);
        d.add(waitForTransition(node, reason => {
            removeClasses(node, ...to, ...base);
            return _done(reason);
        }));
    });
    // Once we get disposed, we should ensure that we cleanup after ourselves. In case of an unmount,
    // the node itself will be nullified and will be a no-op. In case of a full transition the classes
    // are already removed which is also a no-op. However if you go from enter -> leave mid-transition
    // then we have some leftovers that should be cleaned.
    d.add(() => removeClasses(node, ...base, ...from, ...to));
    // When we get disposed early, than we should also call the done method but switch the reason.
    d.add(() => _done(Reason.Cancelled));
    return d.dispose;
}

class TransitionComponent {
    constructor() {
        this._children = new QueryList();
        this._show = new BehaviorSubject(false);
        /**
         * Classes to be applied on the element when showing
         */
        this.enter = '';
        this.enterFrom = '';
        this.enterTo = '';
        /**
         * Classes to be applied on the element when leaving
         */
        this.leave = '';
        this.leaveFrom = '';
        this.leaveTo = '';
        this.beforeEnter = new EventEmitter();
        this.afterEnter = new EventEmitter();
        this.beforeLeave = new EventEmitter();
        this.afterLeave = new EventEmitter();
        this.closed = true;
        this._init = true;
    }
    /**
     * Whether to show or not the contents of ngx-transition
     */
    set show(value) {
        this._show.next(value);
    }
    get show() {
        return this._show.value;
    }
    get getDisplay() {
        return this._init ? 'none' : '';
    }
    ngOnInit() {
        this.closed = !this.show;
        if (this.closed) {
            this._init = true;
            if (this.closed) {
                this.closed = false;
                setTimeout(() => {
                    this._elementRef = this._getElement();
                    this.closed = true;
                });
            }
        }
        else {
            setTimeout(() => {
                this._elementRef = this._getElement();
                this._updateView();
            });
        }
        this._show.pipe(skip(1)).subscribe(() => {
            this._updateView();
        });
    }
    _updateView() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.show) {
                this.closed = false;
                this._transitionEnter();
            }
            else {
                if (this._children.length > 0) {
                    let childrenFinished = new BehaviorSubject(0);
                    for (let child of this._children.toArray().reverse()) {
                        child.afterLeave.pipe(first()).subscribe(() => {
                            childrenFinished.pipe(first()).subscribe((value) => {
                                childrenFinished.next(++value);
                            });
                        });
                        child.show = false;
                    }
                    childrenFinished.subscribe((value) => {
                        if (value === this._children.length)
                            this._transitionLeave();
                    });
                }
                else {
                    this._transitionLeave();
                }
            }
        });
    }
    _getElement() {
        if (!this._elementRef) {
            this._elementRef = this._viewContainer.get(0).rootNodes.find((element) => element.tagName);
            this._init = false;
        }
        return this._elementRef;
    }
    _transitionEnter() {
        this.beforeEnter.emit();
        transition(this._getElement(), this._useSplitClasses(this.enter), this._useSplitClasses(this.enterFrom), this._useSplitClasses(this.enterTo), (reason) => {
            this.afterEnter.emit();
            if (this._children.length > 0) {
                this._children.forEach((child) => {
                    child.show = true;
                });
            }
        });
    }
    _transitionLeave() {
        this.beforeLeave.emit();
        transition(this._getElement(), this._useSplitClasses(this.leave), this._useSplitClasses(this.leaveFrom), this._useSplitClasses(this.leaveTo), (reason) => {
            this.afterLeave.emit();
            this.closed = true;
        });
    }
    _useSplitClasses(classes = '') {
        return classes.split(' ').filter(className => className.trim().length > 1);
    }
}
TransitionComponent.decorators = [
    { type: Component, args: [{
                selector: '[ngx-transition], [transition], transition, ngx-transition',
                template: "<!--\r\n\r\n    <ngx-transition [show]=\"true\"\r\n    [enter]=\"'transition ease-out duration-100'\"\r\n    [enterFrom]=\"'transform opacity-0 scale-95'\"\r\n    [enterTo]=\"'transform opacity-100 scale-100'\"\r\n    [leave]=\"'transition ease-in duration-75'\"\r\n    [leaveFrom]=\"'transform opacity-100 scale-100'\"\r\n    [leaveTo]=\"'transform opacity-0 scale-95'\">\r\n\r\n    </ngx-transition>\r\n\r\n-->\r\n<ng-template #transitionTemplate>\r\n   <ng-content></ng-content>\r\n</ng-template>\r\n\r\n<ng-container #transitionContainer [ngTemplateOutlet]=\"closed ? null : transitionTemplate\"></ng-container>\r\n"
            },] }
];
TransitionComponent.ctorParameters = () => [];
TransitionComponent.propDecorators = {
    _children: [{ type: ContentChildren, args: [TransitionComponent, { descendants: true },] }],
    _viewContainer: [{ type: ViewChild, args: ['transitionContainer', { read: ViewContainerRef, static: true },] }],
    show: [{ type: Input }],
    enter: [{ type: Input }],
    enterFrom: [{ type: Input }],
    enterTo: [{ type: Input }],
    leave: [{ type: Input }],
    leaveFrom: [{ type: Input }],
    leaveTo: [{ type: Input }],
    beforeEnter: [{ type: Output }],
    afterEnter: [{ type: Output }],
    beforeLeave: [{ type: Output }],
    afterLeave: [{ type: Output }],
    getDisplay: [{ type: HostBinding, args: ['style.display',] }]
};

class TransitionModule {
}
TransitionModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TransitionComponent],
                imports: [
                    CommonModule
                ],
                exports: [TransitionComponent]
            },] }
];

/*
 * Public API Surface of ngx-headlessui
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MenuButtonDirective, MenuComponent, MenuItemComponent, MenuItemsDirective, MenuModule, MenuStates, Reason, SwitchComponent, SwitchDescriptionDirective, SwitchGroupDirective, SwitchLabelDirective, SwitchModule, TransitionComponent, TransitionModule, transition };
//# sourceMappingURL=ngx-headlessui.js.map
